"""Quick fastapi script to provide credit simulator"""
import io
import matplotlib

# pylint: disable=wrong-import-position
matplotlib.use("AGG")
import matplotlib.pyplot as plt
from fastapi import FastAPI, Request, Response, BackgroundTasks
from fastapi.responses import HTMLResponse, StreamingResponse
from fastapi.staticfiles import StaticFiles
from jinja2 import Environment, FileSystemLoader
import pandas as pd

import utils as _utils


file_loader = FileSystemLoader("./templates")
env = Environment(loader=file_loader)
env.filters["monnaie"] = _utils.convertir_big_float_vers_jolie_monnaie
app = FastAPI()
app.mount("/img", StaticFiles(directory="img"), name="img")


@app.get("/", response_class=HTMLResponse)
def read_root():
    """Homepage"""
    tpl = env.get_template("home.html")
    output = tpl.render()

    return output


@app.get("/vous-me-pretez-combien", response_class=HTMLResponse)
def read_vous_me_pretez_combien():
    """Vous me prêtez combien ?"""
    tpl = env.get_template("vous-me-pretez-combien.html")
    output = tpl.render()

    return output


@app.get("/vous-me-pretez-combien/resultats", response_class=HTMLResponse)
def read_vous_me_pretez_combien_resultats(request: Request):
    """Vous me prêtez combien ?
    Le résultat
    """
    tpl = env.get_template("vous-me-pretez-combien-resultats.html")

    capital, total_interets = _utils.vous_me_pretez_combien(**request.query_params)

    output = tpl.render(
        capital=capital,
        total_interets=total_interets,
        credit_integral=_utils.credit_integral(**request.query_params),
        **request.query_params,
    )

    return output


@app.get("/je-voudrais-un-million", response_class=HTMLResponse)
def read_je_voudrais_un_million():
    """Je voudrais un million ?"""
    tpl = env.get_template("je-voudrais-un-million.html")
    output = tpl.render()

    return output


@app.get("/je-voudrais-un-million/resultats", response_class=HTMLResponse)
def read_je_voudrais_un_million_resultats(request: Request):
    """Je voudrais un million ?
    Le résultat
    """
    tpl = env.get_template("je-voudrais-un-million-resultats.html")

    paiement_regulier, total_interets = _utils.je_voudrais_un_million(
        **request.query_params
    )

    output = tpl.render(
        paiement_regulier=paiement_regulier,
        total_interets=total_interets,
        credit_integral=_utils.credit_integral(
            paiement_regulier=paiement_regulier, **request.query_params
        ),
        **request.query_params,
    )

    return output


@app.get("/evolution-credit-complet", response_class=HTMLResponse)
def read_evolution_credit_complet():
    """Evolution du crédit complet"""
    tpl = env.get_template("evolution-credit-complet.html")
    output = tpl.render()

    return output


@app.get("/evolution-credit-complet/resultats", response_class=HTMLResponse)
def read_evolution_credit_complet_resultats(request: Request):
    """Evolution du crédit complet
    Le résultat
    """
    tpl = env.get_template("evolution-credit-complet-resultats.html")

    nombre_total_de_paiements, total_interets = _utils.evolution_credit_complet(
        **request.query_params
    )

    output = tpl.render(
        nombre_total_de_paiements=nombre_total_de_paiements,
        total_interets=total_interets,
        credit_integral=_utils.credit_integral(
            nombre_total_de_paiements=nombre_total_de_paiements,
            duree_credit=0,
            **request.query_params,
        ),
        **request.query_params,
    )

    return output


@app.get("/si-je-fais-un-remboursement-anticipe", response_class=HTMLResponse)
def read_si_je_fais_un_remboursement_anticipe(request: Request):
    """Si je fais un remboursement anticipé"""
    tpl = env.get_template("si-je-fais-un-remboursement-anticipe.html")
    output = tpl.render(**request.query_params)

    return output


@app.get("/si-je-fais-un-remboursement-anticipe/resultats", response_class=HTMLResponse)
def read_si_je_fais_un_remboursement_anticipe_resultats(request: Request):
    """Si je fais un remboursement anticipé
    Le résultat
    """
    tpl = env.get_template("si-je-fais-un-remboursement-anticipe-resultats.html")

    credit_integral_sans_anticipation = _utils.credit_integral(
        ignore_anticipation=True, **request.query_params
    )
    credit_integral = _utils.credit_integral(**request.query_params)
    interets_sauves = (
        credit_integral_sans_anticipation[-1]["total_interet"]
        - credit_integral[-1]["total_interet"]
    )
    nb_echeances_sauves = len(credit_integral_sans_anticipation) - len(credit_integral)

    output = tpl.render(
        credit_integral=credit_integral,
        interets_sauves=interets_sauves,
        nb_echeances_sauves=nb_echeances_sauves,
        **request.query_params,
    )

    return output


@app.get("/export/", response_class=StreamingResponse)
async def export_data(request: Request):
    """On export en CSV le résultat du crédit."""
    credit_integral = _utils.credit_integral(**request.query_params)

    liste_pour_csv = _utils.design_credit_integral_pour_export(
        data_list=credit_integral,
        paiement_regulier=request.query_params["paiement_regulier"],
    )

    colonnes = list(liste_pour_csv[0].keys())
    dataframe = pd.DataFrame(
        {colonne: [ligne[colonne] for ligne in liste_pour_csv] for colonne in colonnes}
    )
    stream = io.StringIO()
    dataframe.to_csv(stream, index=False)
    response = StreamingResponse(iter([stream.getvalue()]), media_type="text/csv")
    response.headers["Content-Disposition"] = "attachment; filename=export.csv"
    return response


def create_img_remboursement_cumulatif(data_list: list) -> io.BytesIO:
    """Fonction pour créer le graphique du crédit
    Remboursement cumulatif du capital et des intérêts en fonction des échéances"""
    fig, axis = plt.subplots()
    xy1 = [x["total_interet"] for x in data_list]
    xy2 = [x["total_capital_rembourse"] for x in data_list]
    lines = {
        "intérêt": xy1,
        "capital": xy2,
    }

    axis.stackplot(
        list(range(len(data_list))),
        lines.values(),
        labels=lines.keys(),
        alpha=0.8,
    )
    axis.legend(loc="upper left")
    axis.set_title("Remboursement cumulatif du capital et des intérêts")
    axis.set_xlabel("Échéances")
    axis.set_ylabel("Total du remboursement")

    img_buf = io.BytesIO()
    plt.tight_layout()
    plt.savefig(img_buf, format="png")
    plt.close(fig)
    return img_buf


def create_img_evolution_capital_interet_par_paiement(data_list: list) -> io.BytesIO:
    """Fonction pour créer le graphique du crédit
    Répartition capital/intérêt à chaque paiement en fonction des échéances"""
    fig, axis = plt.subplots()
    xy1 = [x["remboursement_interet"] for x in data_list]
    xy2 = [x["remboursement_capital"] for x in data_list]
    lines = {
        "intérêt": xy1,
        "capital": xy2,
    }

    axis.stackplot(
        list(range(len(data_list))),
        lines.values(),
        labels=lines.keys(),
        alpha=0.8,
    )
    axis.legend(loc="upper right")
    axis.set_title("Remboursement du capital et des intérêts")
    axis.set_xlabel("Échéances")
    axis.set_ylabel("Remboursement")

    img_buf = io.BytesIO()
    plt.tight_layout()
    plt.savefig(img_buf, format="png")
    plt.close(fig)
    return img_buf


@app.get("/graphique/cumulatif")
def get_graphique_cumulatif(background_tasks: BackgroundTasks, request: Request):
    """Dessiner le graphique du crédit en cumulatif"""
    credit_integral = _utils.credit_integral(**request.query_params)

    img_buf = create_img_remboursement_cumulatif(data_list=credit_integral)
    background_tasks.add_task(img_buf.close)
    headers = {"Content-Disposition": 'inline; filename="cumulatif.png"'}
    return Response(img_buf.getvalue(), headers=headers, media_type="image/png")


@app.get("/graphique/unitaire")
def get_graphique_unitaire(background_tasks: BackgroundTasks, request: Request):
    """Dessiner le graphique du crédit en unitaire"""
    credit_integral = _utils.credit_integral(**request.query_params)

    img_buf = create_img_evolution_capital_interet_par_paiement(
        data_list=credit_integral
    )
    background_tasks.add_task(img_buf.close)
    headers = {"Content-Disposition": 'inline; filename="unitaire.png"'}
    return Response(img_buf.getvalue(), headers=headers, media_type="image/png")
