"""Toutes les fonctions pour jouer avec le crédit"""
from typing import Union
from math import log, ceil
import locale


# pylint: disable=too-many-arguments
def vous_me_pretez_combien(
    duree_credit: str,
    taux_interet: str,
    frequence: Union[str, int],
    paiement_regulier: str,
    nb_paiements_faits: int = 0,
    nombre_total_de_paiements: int = None,
) -> tuple:
    """Cette fonction nous dit combien de capital on peut avoir
    si on a un taux fixe de :taux_interet pendant :duree_credit ans
    et si on paie :paiement_regulier :frequence par an
    on ajoute aussi les intérêts qu'on paie

    On peut y enlever un nb_paiements_faits
    """
    if not isinstance(frequence, int):
        frequence = convertir_frequence_vers_entier(frequence)

    # on peut fournir les deux options
    if not nombre_total_de_paiements:
        duree_credit = int(duree_credit)
        nombre_total_de_paiements = duree_credit * frequence
    else:
        duree_credit = 0

    taux_interet = float(taux_interet)
    paiement_regulier = float(paiement_regulier)

    nombre_total_de_paiements -= nb_paiements_faits

    facteur_un = paiement_regulier * frequence / (taux_interet / 100)

    facteur_deux = 1 - (1 + (taux_interet / 100 / frequence)) ** (
        -nombre_total_de_paiements
    )

    capital = facteur_un * facteur_deux
    total_interets = (nombre_total_de_paiements * paiement_regulier) - capital

    return (capital, total_interets)


def convertir_frequence_vers_entier(frequence_str: str) -> int:
    """On veut convertir en entier"""
    if frequence_str == "tous_les_mois":
        return 12
    if frequence_str == "deux_fois_par_mois":
        return 24
    if frequence_str == "toutes_les_deux_semaines":
        return 26
    raise ValueError(f"{frequence_str} n'est pas un cas connu")


def convertir_big_float_vers_jolie_monnaie(money: float) -> str:
    """Transformer le gros float en quelque chose de beau"""
    if not isinstance(money, float):
        money = float(money)
    locale.setlocale(locale.LC_ALL, "")
    return locale.currency(money, grouping=True)


def je_voudrais_un_million(
    duree_credit: str,
    taux_interet: str,
    frequence: str,
    capital: str,
) -> tuple:
    """Cette fonction nous dit combien on va payer :frequence par an
    si on a un taux fixe de :taux_interet pendant :duree_credit ans
    et si on souhaite un capital de :capital
    on ajoute aussi les intérêts qu'on paie
    """

    frequence = convertir_frequence_vers_entier(frequence)
    duree_credit = int(duree_credit)
    taux_interet = float(taux_interet) / 100
    capital = float(capital)

    nombre_total_de_paiements = duree_credit * frequence

    paiement_regulier = (capital * taux_interet / frequence) / (
        1 - (1 + (taux_interet / frequence)) ** -nombre_total_de_paiements
    )
    total_interets = (nombre_total_de_paiements * paiement_regulier) - capital

    return (paiement_regulier, total_interets)


def evolution_credit_complet(
    taux_interet: str,
    frequence: str,
    capital: str,
    paiement_regulier: str,
) -> tuple:
    """Cette fonction nous dit combien on va payer :frequence par an
    si on a un taux fixe de :taux_interet pendant :duree_credit ans
    et si on souhaite un capital de :capital
    on ajoute aussi les intérêts qu'on paie
    """

    if not isinstance(frequence, int):
        frequence = convertir_frequence_vers_entier(frequence)
    taux_interet = float(taux_interet) / 100
    capital = float(capital)
    paiement_regulier = float(paiement_regulier)

    nombre_total_de_paiements = -(
        log(1 - ((capital * taux_interet) / (frequence * paiement_regulier)))
    ) / (log(1 + (taux_interet / frequence)))

    total_interets = (nombre_total_de_paiements * paiement_regulier) - capital

    return (nombre_total_de_paiements, total_interets)


# pylint: disable=unused-argument,too-many-locals
def credit_integral(
    taux_interet: str,
    frequence: str,
    paiement_regulier: str,
    duree_credit: str,
    nombre_total_de_paiements: int = None,
    paiement_anticipe: str = None,
    echeance_paiement_anticipe: str = None,
    ignore_anticipation: bool = False,
    **kwargs: dict,
) -> list:
    """Cette fonction fait le calcul complet du credit
    et retourne une liste exploitable en CSV
    on gère aussi l'option d'un :paiement_anticipe à :echeance_paiement_anticipe
    """

    frequence = convertir_frequence_vers_entier(frequence)
    # on peut fournir les deux options
    if not nombre_total_de_paiements:
        duree_credit = int(duree_credit)
        nombre_total_de_paiements = duree_credit * frequence
    else:
        duree_credit = 0
        if not isinstance(nombre_total_de_paiements, int):
            nombre_total_de_paiements = ceil(float(nombre_total_de_paiements))

    taux_interet = float(taux_interet)
    paiement_regulier = float(paiement_regulier)

    total_paiement = 0
    total_interet = 0
    total_capital_rembourse = 0

    res = []

    nb_paiements_faits = 0

    while (
        nb_paiements_faits < nombre_total_de_paiements
        and float(kwargs.get("capital", "inf")) > total_capital_rembourse
    ):
        capital_restant = vous_me_pretez_combien(
            duree_credit=duree_credit,
            taux_interet=taux_interet,
            frequence=frequence,
            paiement_regulier=paiement_regulier,
            nb_paiements_faits=nb_paiements_faits + 1,
            nombre_total_de_paiements=nombre_total_de_paiements,
        )[0]

        capital_restant_precedent = vous_me_pretez_combien(
            duree_credit=duree_credit,
            taux_interet=taux_interet,
            frequence=frequence,
            paiement_regulier=paiement_regulier,
            nb_paiements_faits=nb_paiements_faits,
            nombre_total_de_paiements=nombre_total_de_paiements,
        )[0]

        remboursement_capital = capital_restant_precedent - capital_restant
        remboursement_interet = paiement_regulier - remboursement_capital

        if paiement_anticipe and echeance_paiement_anticipe and not ignore_anticipation:
            # cas du remboursement anticipé
            if int(echeance_paiement_anticipe) == nb_paiements_faits + 1:
                capital_restant = capital_restant - float(paiement_anticipe)
                total_paiement += float(paiement_anticipe)
                # la durée du crédit va diminuer ainsi que le nombre total de paiement
                nombre_total_de_paiements = (
                    evolution_credit_complet(
                        taux_interet=taux_interet,
                        frequence=frequence,
                        capital=capital_restant,
                        paiement_regulier=paiement_regulier,
                    )[0]
                    + nb_paiements_faits
                )

        total_paiement += paiement_regulier
        total_interet += remboursement_interet
        total_capital_rembourse = total_paiement - total_interet

        res.append(
            {
                "remboursement_capital": remboursement_capital,
                "remboursement_interet": remboursement_interet,
                "total_paiement": total_paiement,
                "total_interet": total_interet,
                "total_capital_rembourse": total_capital_rembourse,
            }
        )

        nb_paiements_faits += 1

    return res


def design_credit_integral_pour_export(data_list: list, paiement_regulier: str) -> list:
    """On met au propre la liste pour export CSV"""
    resultat = []

    for index, data in enumerate(data_list):
        if index == 0:
            montant_paye = paiement_regulier
        else:
            montant_paye = (
                data["total_paiement"] - data_list[index - 1]["total_paiement"]
            )
        resultat.append(
            {
                "# échéance": index + 1,
                "Montant payé": montant_paye,
                "Remboursement du capital": data["remboursement_capital"],
                "Intérêt payé": data["remboursement_interet"],
                "Total montant payé": data["total_paiement"],
                "Total remboursement du capital": data["total_capital_rembourse"],
                "Total intérêt payé": data["total_interet"],
            }
        )

    return resultat
